<?php

    class File_Operations_Action extends Reactor_Action
    {
        protected $tpl = 'file_operations.php';
        protected $name = 'Perform a file or directory operation';
		
		public static $registration = array(
			ACTION_NAME => 'File operations',
			ACTION_CATEGORY => 'Local',
			ACTION_DESCRIPTION => 'Perform one of several file operations',
			ACTION_AUTHOR => 'Jeremy Yunis, Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '03/01/2013', //Last version update
			ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			ACTION_TYPE => 'core', //core | user. core modules can't be deleted
			ACTION_META => array(),	//Misc information	
		); 

        protected $fields = array(
            'action' => array(
                'label' => 'action',
                'validate' => 'trim|required'
            ),		
            'perms' => array(
                'label' => 'perms',
                'validate' => 'trim|numeric|min_length[4]|max_length[4]'
            ),		
            'newname' => array(
                'label' => 'newname',
                'validate' => 'trim|callback_check_filename|xss_clean'
            ),			
            'filename' => array(
                'label' => 'filename',
                'validate' => 'trim|callback_check_filename'
            ),			

        );
		
        public function check_owner($str)
        {
			if($str=='')
				return TRUE; 

			/* TODO - validate user:group */
			return TRUE;
        }

			
        public function check_filename($str)
        {
			if($str=='')
				return TRUE; 
		
            $banned_files = array(
                '/etc/passwd',
                '/etc/hosts',
                '/etc/resolv.conf',
                '/etc/network/interfaces',
                '/etc/sysconfig/network',
            );

            // check for path traversal (dot slash)
            $exp = '/(\.\/)/';
            if(preg_match($exp, $str)) {
                $this->ci->form_validation->set_message('check_filename', 'The filename must be absolute.');
                return FALSE;
            }

            // check for a generally acceptable file
            $exp = '/^\/([^\0]+\/)+([^\0]+)$/';
            if(!preg_match($exp, $str)) {
                $this->ci->form_validation->set_message('check_filename', _('The filename was invalid. Please ensure you have entered a valid, absolute UNIX filename and path.'));
                return FALSE;
            }
                 
            // check against banned list
            foreach($banned_files as $file) {
                if($file == $str) {
                    $this->ci->form_validation->set_message('check_filename', _('Writing to the file specified is not allowed.'));
                    return FALSE;
                }
            }
            return TRUE;
        }

        public function get_label()
        {
			if($params = $this->get_params()){
			
				$file = $this->get_param('filename');
				$action = $this->get_param('action');
				$perms = $this->get_param('perms');
				$owner = $this->get_param('owner');
				$newname = $this->get_param('newname');
				
				$xstr="";
				
				switch($action) {
					case 'create_dir':
						$astr="Create directory"; 
						break; 

					case 'change_permission':
						$astr="Change permissions on file";
						$xstr="to '".$perms."'";
						break;
					
					case 'touch_file':
						$astr="Touch file";
						break;
						
					case 'rename_file':
						$astr="Rename file";
						$xstr="to '".$newname."'";
						break; 
					
					case 'delete_file':
						$astr="Delete file";
						break;
					
					default:
						$astr="?";
						break;				
					}
				
				return sprintf(_('%s \'%s\' %s'), $astr, $file, $xstr);
				/*
				if(isset($params['filename'],$params['action'])) {
					return sprintf(_('Perform file action: %s on file: %s'), $params['action'], $params['filename']);
					}
				*/
				}
            return parent::get_label();
        }

        public function run($context)
        {
            $this->set_context($context);
            $params = $this->get_params();
            $file = $this->get_param('filename');
			$action = $this->get_param('action');
			$perms = $this->get_param('perms');
			$newname = $this->get_param('newname');
			
			$result=false;
			
			switch($action) {
				case 'create_dir':
					$result = $this->create_dir($file); 
				break; 
				
				case 'change_permission':
					$result = $this->change_permissions($file,$perms);
				break;
				
				case 'touch_file':
					$result = $this->touch_file($file);
				break;
					
				case 'rename_file':
					$result = $this->rename_file($file,$newname);
				break; 
				
				case 'delete_file':
					$result = $this->delete_file($file);
				break;
				
				default:
					$this->log('error', 'Illegal action. Method does not exist!');
				break;				
			}

			if(!$result) {
				$this->log('error', 'File operation: '.$action. ' FAILED for '.$file.'. Check permissions.');
				$this->set_result(1);
			}	
			else {
				$this->log('info', 'File operation: '.$action. ' succeeded for '.$file);
				$this->set_result(0);
			}
            
            return $context;
        }
    
	
	/*
	- Create directory
	- Touch (create) file
	
	- Set directory permissions
	- Set directory ownership
	- Delete directory
	- Rename directory
	
	- Set file permissions
	- Delete file
	- Rename file
		*/	
	
	private function create_dir($file) {
		return @mkdir($file);
	}
	
	
	private function change_permissions($file, $perms) {
		if (is_file($file)) {
			$cmd = "/bin/chmod " . $perms . " " . $file;
			@exec($cmd, $output, $code);

			if ($code == 0) {
				return true;
			}
		}

		return false;
	}
	
	private function touch_file($file) {
		return @touch($file); 
	}
	
	private function rename_file($file,$newname) {
		return @rename($file,$newname); 
	}
	
	private function delete_file($file) {		
		if(is_dir($file)) {
			@exec('/bin/rm -rf '.$file,$output,$code);
			if($code==0)
				return true;
			else
				return false;
		}
		else 
			return @unlink($file); 
	}
	
	
}	
	
	
